


auc <- function(sens, spec)
{
.tab.res<-data.frame(se=sens, sp=spec)
.tab.res<-.tab.res[!is.na(.tab.res$sp + .tab.res$se),]
.tab.res$sp1 <- 1-.tab.res$sp
.tab.res<-.tab.res[order(.tab.res$sp1, .tab.res$se),]

return( sum((.tab.res$sp1[2:length(.tab.res$sp1)] -
  .tab.res$sp1[1:(length(.tab.res$sp1)-1)]) *
  (.tab.res$se[2:length(.tab.res$se)])) )
}

ROCt632 <- function(times, failures, features, N.boot, precision, prop, pro.time)
{

.E <- features

.G <- dim(features)[2]

.N <- dim(features)[1]

.Data <- data.frame(ident = 1:length(times), t=times, f=failures)

cut.off  <- qnorm(precision, mean = 0, sd = 1, lower.tail = TRUE, log.p = FALSE)

auc.train <- rep(-99, length(pro.time))
auc.valid <- rep(-99, length(pro.time))
auc.632 <- rep(-99, length(pro.time))
auc.632p <- rep(-99, length(pro.time))

k <-1

for (prognostic.time in pro.time)
{

cat("# year ", k+1, "\n", sep="")

B.train <- matrix(-99, nrow=N.boot, ncol=.G)

sd.train <- rep(-99, N.boot)
mean.train <- rep(-99, N.boot)

fn.train <- matrix(-99, nrow=N.boot, ncol=length(precision))
fp.train <- matrix(-99, nrow=N.boot, ncol=length(precision))

fn.valid<-matrix(-99, nrow=N.boot, ncol=length(precision))
fp.valid<-matrix(-99, nrow=N.boot, ncol=length(precision))

i <- 1

while (i <= N.boot) {
 
num <- sample(.Data$ident, size = .N, replace = TRUE)

# Calcul sur l'chantillon de Bootstrap (train)

Eb <- .E[num,] ; Data.b <- .Data[num,]

opt1 <- try(optL1(Surv(Data.b$t, Data.b$f), penalized = Eb,
 fold=5, trace = FALSE), silent = FALSE)

if(inherits(opt1, "try-error")) { i <- i; cat(" test0 ") } else {

pen1 <- try(penalized(Surv(Data.b$t, Data.b$f), penalized = Eb,
 lambda1=opt1$lambda, trace = FALSE), silent = FALSE)

if(inherits(pen1, "try-error")) { i <- i; cat(" test1 ") } else {

B.train[i,]  <- coefficients(pen1, "all")

if(sum(B.train[i,])==0)
{
fn.train[i,]<-precision
fp.train[i,]<-1-precision
fn.valid[i,]<-precision
fp.valid[i,]<-1-precision
i <- i + 1
} else {

Data.b$S.l1 <- Eb %*% B.train[i,]

sd.train[i] <- sd(Data.b$S.l1)

mean.train[i] <- mean(Data.b$S.l1)

Data.b$S.l1 <- (Data.b$S.l1 - mean.train[i]) / sd.train[i]

ROCt.l1 <- try(survivalROC(Stime = Data.b$t, status = Data.b$f, marker = Data.b$S.l1, predict.time = prognostic.time, cut.values = cut.off, method = "NNE", lambda = prop), silent = TRUE)

if(inherits(ROCt.l1, "try-error")) { i <- i; cat(" test2 ") } else {

if(length(ROCt.l1$TP)==0 | is.na(sum(ROCt.l1$TP[-1])) | is.na(sum(ROCt.l1$FP[-1])))
 { i <- i } else {
 
fn.train[i,]<-1-ROCt.l1$TP[-1]
fp.train[i,]<-ROCt.l1$FP[-1]

# Calcul sur l'chantillon de validation (valid)

Eb <- .E[-unique(sort(num)),] ; Data.b <- .Data[-unique(sort(num)),]

Data.b$S.l1 <- Eb %*% B.train[i,]
Data.b$S.l1 <- (Data.b$S.l1 - mean.train[i]) / sd.train[i]

ROCt.l1 <- try(survivalROC(Stime = Data.b$t, status = Data.b$f, marker = Data.b$S.l1, predict.time = prognostic.time, cut.values = cut.off, method = "NNE", lambda = prop), silent = TRUE)

if(inherits(ROCt.l1, "try-error")) { i <- i; cat(" test3 ") } else {

if(length(ROCt.l1$TP)==0 | is.na(sum(ROCt.l1$TP[-1])) | is.na(sum(ROCt.l1$FP[-1])))
 { i <- i; cat(" test4 ") } else {
 
fn.valid[i,]<-1-ROCt.l1$TP[-1]
fp.valid[i,]<-ROCt.l1$FP[-1]

cat("   sample ", i, "\n", sep="")

i <- i + 1

} } } } } } } }

# Calcul des estimateurs "0.632"

mean.fn.train <- apply(fn.train, FUN="mean", MARGIN=2)
mean.fp.train <- apply(fp.train, FUN="mean", MARGIN=2)

mean.fn.valid <- apply(fn.valid, FUN="mean", MARGIN=2)
mean.fp.valid <- apply(fp.valid, FUN="mean", MARGIN=2)

fn.632 <- 0.368 * mean.fn.train + 0.632 * mean.fn.valid
fp.632 <- 0.368 * mean.fp.train + 0.632 * mean.fp.valid

r.fn <- (mean.fn.valid - mean.fn.train)/(precision - mean.fn.train)
r.fp <- (mean.fp.valid - mean.fp.train)/((1-precision) - mean.fp.train)

r.fn <- pmax(pmin(r.fn, 1), 0)
r.fp <- pmax(pmin(r.fp, 1), 0)

fn.632p<-(1-(0.632/(1-0.368*r.fn)))*mean.fn.train + (0.632/(1-0.368*r.fn)) * mean.fn.valid
fp.632p<-(1-(0.632/(1-0.368*r.fp)))*mean.fp.train + (0.632/(1-0.368*r.fp)) * mean.fp.valid

auc.train[k] <- auc(sens = 1-mean.fn.train, spec = 1-mean.fp.train)
auc.valid[k] <- auc(sens = 1-mean.fn.valid, spec = 1-mean.fp.valid)
auc.632[k]   <- auc(sens = 1-fn.632, spec = 1-fp.632)
auc.632p[k]  <- auc(sens = 1-fn.632p, spec = 1-fp.632p)

k<-k+1

}

return( data.frame(pro.time=pro.time, train=auc.train,
 valid=auc.valid, s632=auc.632, p632=auc.632p) )

}
